

import java.io.IOException;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;

import com.mascotcapsule.micro3d.v3.AffineTrans;
import com.mascotcapsule.micro3d.v3.Effect3D;
import com.mascotcapsule.micro3d.v3.Figure;
import com.mascotcapsule.micro3d.v3.FigureLayout;
import com.mascotcapsule.micro3d.v3.Graphics3D;
import com.mascotcapsule.micro3d.v3.Light;
import com.mascotcapsule.micro3d.v3.Vector3D;


public class CanvasEx extends GameCanvas implements CommandListener, Runnable{
	/**
	 * 変数
	 */
	/* MascotCapsule 基本オブジェクト */
	private Figure figure;				// モデルデータ
	
	/* MascotCapsule 描画効果、座標変換関係オブジェクト */
	private Effect3D effect;			// 描画効果
	private FigureLayout layout = new FigureLayout();	//座標変換関係
	
	/* MascotCapsule 光源設定用パラメータ */
	private Vector3D lightDir = new Vector3D( -4096, 4096, 0 );
	private int ambIntensity = 4096 * 1 / 4;
	private int dirIntensity = 4096 * 3 / 4;
	private Light light = new Light( lightDir, dirIntensity, ambIntensity );
	
	/* MascotCapsule 回転制御用パラメータおよび各変換行列 */
	private AffineTrans vtrans = new AffineTrans(); // 視点変換行列
	
	/* スクリーン設定用パラメータ */
	private int centerX = getWidth() >> 1;				// スクリーン中心
	private int centerY = getHeight() >> 1;
	
	/* 背景色 */
	private int bgc = 0x003399;
	int speed = 1;
	
	/* その他 */
	private String commandLabels[] = {"UP", "DOWN"}; // ソフトキーラベル
	
	Graphics3D g3d = new Graphics3D();
	Graphics graphics;
	
	/**
	 * CanvasEx
	 * コンストラクタ(データロード等初期化）
	 */
	public CanvasEx(){
		super(true);
		try{
			figure = new Figure( "/sphere.mbac" );
		}catch( IOException e ){
			e.printStackTrace();
		}

		effect = new Effect3D( light, Effect3D.NORMAL_SHADING, true, null );
		layout.setPerspective( 512, 4096, 4096 * 30 / 360 );
		
		addCommand( new Command( commandLabels[0], Command.SCREEN, 1));
		addCommand( new Command( commandLabels[1], Command.SCREEN, 1));
		setCommandListener( this );
	}
	
	/**
	 * run() メソッド
	 * Thread 処理本体
	 */
	public void run(){
		int i = 0;
		boolean plus = true;
		for(;;){		// 永久ループ
			
			/* スクリーン設定 */
			double r = 80.0;
			int x = (int)(Math.sin((i / 180.0) * 3.14) *  r);
			int y = (int)(Math.cos((i / 180.0) * 3.14) *  r);
			layout.setCenter( centerX + x, centerY + y);
			i += speed;
			if(i >= 360)
				i = 360 - i;
			vtrans.lookAt( new Vector3D( 0, 0, 1024 + ((int)r-y)*8),
							  new Vector3D( 0, 0, -4096 ),
							  new Vector3D( 0, 4096, 0 ));	// 視点変換行列

			/* 描画要求＋sleep(100ms) */
			graphics = this.getGraphics();
			paint( graphics );
			flushGraphics();
			try{
				Thread.sleep( 1 );
			}catch( InterruptedException ie ){
				
			}
		}
	}

	/**
	 * paint() メソッド
	 * 画面クリア、クリッピング領域設定/クリア、3D描画
	 * 
	 * @param g Graphicsインスタンス
	 */
	public void paint( Graphics g ) {
		g.setColor( bgc );		// 背景色（全体）
		g.fillRect( 0, 0, getWidth(), getHeight() );
		layout.setAffineTrans( vtrans );
		
		g3d.bind( g );
		g3d.renderFigure( figure, 0, 0, layout, effect );
		g3d.flush();
		g3d.release( g );
	}

	
	/**
	 * commandAction() メソッド
	 * ソフトキー押下時処理（描画パラメータ変更）
	 */
	public void commandAction( Command c, Displayable d ){
		String label = c.getLabel();
		// speed 1,2,4
		if( label.equals( commandLabels[0] ) ){
			if(speed != 4)
				speed = speed << 1;
		}else if( label.equals( commandLabels[1] ) ){
			if(speed != 1)
				speed = speed >> 1;				
		}
	}
}
